#include "./USB/usb.h"
#include "./USB/usb_function_cdc.h"

#include "GenericTypeDefs.h"
#include "Compiler.h"
#include "usb_config.h"
#include "USB\usb_device.h"
#include "USB\usb.h"

#include "HardwareProfile.h"

#include "HardwareProfile-CDC-W509.h"
#include <pwm.h>
#include <delays.h>

#include "user.h"


#pragma udata
char USB_In_Buffer[64];
char USB_Out_Buffer[64];

BOOL stringPrinted;

unsigned char CMND1 = 0xFF;
unsigned char CMND[3];
unsigned char TURN = 0;
unsigned char iBuf = 0;

#define MAX_RRPT 10
unsigned char iRpt;

/******************************************************************************
 * Function:        void UserInit(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This routine should take care of all of the demo code
 *                  initialization that is required.
 *
 * Note:            
 *
 *****************************************************************************/
void UserInit(void)
{
	char period=0x00;
//	unsigned char outputconfig=0,outputmode=0,config=0;
	unsigned int duty_cycle=0;

    stringPrinted = TRUE;
    //Initialize all of the LED pins

	mInitAllLEDs();
    mInitAllSwitches();

	// My init
	mInitW509Ports();

	// Set-up PWM
	// PWM output = RC4, RC5
	TRISC &= 0b11001111;

	//----Configure pwm ----
    period = 0xFF;
    OpenPWM1( period);            //Configure PWM module and initialize PWM period

	//-----set duty cycle----
    duty_cycle = 0x0200;	// 50%
    SetDCPWM1(duty_cycle);        //set the duty cycle

	//----set pwm output----
	mSet_PWM_Mode();

}//end UserInit

/********************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user
 *                  routines. It is a mixture of both USB and
 *                  non-USB tasks.
 *
 * Note:            None
 *******************************************************************/
void UserMain(void)
{   
    BYTE numBytesRead;
	unsigned char Cmnd2;
	unsigned short s_Cmnd2;
	unsigned short RcvData;

    if(mUSBUSARTIsTxTrfReady())
    {
		numBytesRead = getsUSBUSART(&USB_Out_Buffer[iBuf],50);

		if(numBytesRead>0)
			switch(CMND1)
			{
				case 0x00:
					Cmnd2 = USB_Out_Buffer[0];
					Command_00(Cmnd2);
					CMND1 =0xFF;
					break;
				case 0x01:
					//(01)Single Command
					Cmnd2 = USB_Out_Buffer[0];
					SendCmnd6(Cmnd2);
					USB_In_Buffer[0] = Cmnd2;
					putUSBUSART(USB_In_Buffer,1);
					CMND1 =0xFF;
					break;
				case 0x02:
					//(02)Receive Data
					Cmnd2 = USB_Out_Buffer[0];
					RcvData = RcvData1(Cmnd2);
					USB_In_Buffer[0] = Cmnd2;
					USB_In_Buffer[1] = (unsigned char)RcvData;
					RcvData = RcvData >> 8;
					USB_In_Buffer[2] = (unsigned char)RcvData;
					putUSBUSART(USB_In_Buffer,3);
					CMND1 =0xFF;
					break;
				case 0x03:
					//(03)Internal Timing
					CMND[TURN] = USB_Out_Buffer[0];
					USB_In_Buffer[0] = USB_Out_Buffer[0];
					TURN++;
					if(TURN>=2)
					{
						SendCmnd6(CMND[0]);
						mDelay100usx(CMND[1]);
						CMND1 =0xFF;
						TURN = 0;
					}
					putUSBUSART(USB_In_Buffer,1);
					break;
				case 0x04:
					//(04)Send Data
					CMND[TURN] = USB_Out_Buffer[0];
					USB_In_Buffer[0] = USB_Out_Buffer[0];
					TURN++;
					if(TURN>=3)
					{
						Command_04();
						CMND1 =0xFF;
						TURN = 0;
					}
					putUSBUSART(USB_In_Buffer,1);
					break;
				case 0x05:
					//(05)External Timing
					CMND[TURN] = USB_Out_Buffer[0];
					USB_In_Buffer[0] = USB_Out_Buffer[0];
					TURN++;
					if(TURN>=3)
					{
						Command_05();
						CMND1 =0xFF;
						TURN = 0;
					}
					putUSBUSART(USB_In_Buffer,1);
					break;
				case 0x06:
					//(06)Single Command x N
					CMND[TURN] = USB_Out_Buffer[0];
					USB_In_Buffer[0] = USB_Out_Buffer[0];
					TURN++;
					if(TURN>=3)
					{
						Command_06();
						CMND1 =0xFF;
						TURN = 0;
					}
					putUSBUSART(USB_In_Buffer,1);
					break;
				case 0x07:
					//(07)Receive Data x N
					CMND[TURN] = USB_Out_Buffer[0];
					USB_In_Buffer[0] = USB_Out_Buffer[0];
					putUSBUSART(USB_In_Buffer,1);
					TURN++;
					if(TURN>=3)
					{
						Command_07();
						CMND1 =0xFF;
						TURN = 0;
					}
					break;
				case 0x08:
					//(09)Send20
					iBuf += numBytesRead;
					if(iBuf >= 13)
					{
						Command_08();
						CMND1 =0xFF;
						iBuf = 0;
					}
					break;
				case 0x09:
					//(08)Advanced Write 4
					iBuf += numBytesRead;
					if(iBuf >= 5)
					{
						Command_09();
						CMND1 =0xFF;
						iBuf = 0;
					}
					break;
				case 0x0A:
					//(0A)Receive8
					Cmnd2 = USB_Out_Buffer[0];
					USB_In_Buffer[0] = RcvData18(Cmnd2);
					putUSBUSART(USB_In_Buffer,1);
					CMND1 =0xFF;
					break;
				case 0x0B:
					//(0B)Address Set24
					iBuf += numBytesRead;
					if(iBuf >= 3)
					{
						Command_0B();
						CMND1 =0xFF;
						iBuf = 0;
					}
					break;
				case 0x0C:
					//(0C)Table Read N-Byte
					iBuf += numBytesRead;
					if(iBuf >= 2)
					{
						Command_0C();
						CMND1 =0xFF;
						iBuf = 0;
					}
					break;
				case 0x0D:
					//(0D)Write Data 8Byte
					iBuf += numBytesRead;
					if(iBuf >= 9)
					{
						Command_0D();
						CMND1 =0xFF;
						iBuf = 0;
					}
					break;
				case 0x0E:
					//(0E)EEPROM Address Set
					iBuf += numBytesRead;
					if(iBuf >= 2)
					{
						Command_0E();
						CMND1 =0xFF;
						iBuf = 0;
					}
					break;
				case 0x0F:
					//(0F)EEPROM Read N-Byte
					iBuf += numBytesRead;					
					if(iBuf>=2) 
					{
						s_Cmnd2 = USB_Out_Buffer[1];
						s_Cmnd2 = s_Cmnd2 << 8;
						s_Cmnd2 += USB_Out_Buffer[0];
						Command_0F(s_Cmnd2);
						CMND1 =0xFF;
						iBuf = 0;
					}
					break;
				case 0x10:
					//(10)EEPROM Write 1Byte
					Cmnd2 = USB_Out_Buffer[0];
					Command_10(Cmnd2);
					CMND1 =0xFF;
					break;
				case 0xFF:
					CMND1 = USB_Out_Buffer[0];
					TURN = 0;
					iBuf = 0;
					USB_In_Buffer[0] = CMND1;
					iRpt = 0;
					putUSBUSART(USB_In_Buffer,1);
					break;
				default:
					CMND1 = 0xFF;
					TURN = 0;
					iBuf = 0;
					break;
			}//End switch
		//if(numBytesRead>0)
/*
		else if(CMND1==0x0F && iBuf==1)
    	{
			iRpt++;
			if(iRpt>MAX_RRPT)
			{
//				if(iBuf==1)
//				{
					s_Cmnd2 = USB_Out_Buffer[0];
					if(s_Cmnd2==0)
						s_Cmnd2 = 256;
					Command_0F(s_Cmnd2);
				}
				CMND1 =0xFF;
				iBuf = 0;
//			}

		}
*/

	}//End if(mUSBUSARTIsTxTrfReady())



    CDCTxService();
}		//end ProcessIO

//	Function of Writer509void 
void Command_00(unsigned char Cmnd2)
{
	//(00-00)Version
	if(Cmnd2 == 0x00)
	{
		rom char* VerData = "xW509 Ver.1.24";
		unsigned char i;
		USB_In_Buffer[0] = 0x00;
		for(i=1;i<14;i++)
		{
			USB_In_Buffer[i] = VerData[i];
		}
		putUSBUSART(USB_In_Buffer,14);

		//putrsUSBUSART("W509 Ver.1.24");
	}
	else
	{
		switch(Cmnd2)
		{
			case 0x01:
				//(00-01)Vdd,Vpp-ON
				mVdd_on();
				mDelay10u();
				mVpp_on();
				USB_In_Buffer[0] = 0x01;
				break;
			case 0x02:
				//(00-02)Vpp,Vdd-OFF
				mVpp_off();
				mDelay10u();
				mVdd_off();
				USB_In_Buffer[0] = 0x02;
				break;
			case 0x03:
				//(00-03)Vpp,Vdd-ON
				mVpp_on();
				mDelay10u();
				mVdd_on();
				USB_In_Buffer[0] = 0x03;
				break;
			case 0x04:
				//(00-04)Vdd,Vpp-OFF
				mVdd_off();
				mDelay10u();
				mVpp_off();
				USB_In_Buffer[0] = 0x04;
				break;
		}// End switch
		putUSBUSART(USB_In_Buffer,1);
	}// End else
}//End of Command_00()

void SendCmnd6(unsigned char Cmnd)
{
	// Send Command(6bit) to the device
	int i;
	mPGC_L();
	mPGD_L();
	for(i=0; i<6; i++)
	{
		mPGC_H();
		if(Cmnd & 0x01)
		{
			mPGD_H();
		}
		else
		{
			mPGD_L();
		}
		mDelayPGC();
		mPGC_L();
		mDelayPGC();
		Cmnd = Cmnd >> 1;
	}//end for
	mPGD_L();
	mDelayTdly1();
}// End SendCmnd6()

void SendData14(unsigned short Data)
{
	// Send Data(14bit) to the device
	unsigned char i;
	mPGC_L();
	mPGD_L();
	Data = Data << 1;
	for(i=0; i<16; i++)
	{
		mPGC_H();
		if(Data & 0x0001)
		{
			mPGD_H();
		}
		else
		{
			mPGD_L();
		}
		mDelayPGC();
		mPGC_L();
		mDelayPGC();
		Data = Data >> 1;
	}//end for
	mPGD_L();
	mDelayTdly1();
}//SendData14()


unsigned short RcvData1(unsigned char Cmnd)
{
	// Recieve Data(14bit) from the device after sending a read command(6bit)
	unsigned char i;
	unsigned short RcvData = 0;
	SendCmnd6(Cmnd);
	mTRIS_PGD = 1;	// Change Port_PGD to Input
	mPGC_L();
//	mDelayTdly1();

	for(i=0; i<16; i++)
	{
		mPGC_H();
		mDelayPGC();

//		mPGC_L();
		if(mPort_PGD)
		{
			RcvData += 0x8000;
		}
		mPGC_L();
		mDelayPGC();
		RcvData = RcvData >> 1;

	}//End for

	mTRIS_PGD = 0;	// Change Port_PGD to Output
	mDelayTdly1();
	return RcvData;	
}

//void Command_03(void)
//{
	//(03)Internal Timing

//	SendCmnd6(CMND[0]);
//	mDelay100usx(CMND[1]);

//}//End of Command_03()

void Command_04(void)
{
	//Send Data
	unsigned short SndData;

	SndData = CMND[2];
	SndData = SndData << 8;
	SndData += CMND[1];

	SendCmnd6(CMND[0]);
//	mDelayTdly1();
	SendData14(SndData);
}//End of Command_04()

void Command_05(void)
{
	//(05)External Timing
	SendCmnd6(CMND[0]);
	mDelay100usx(CMND[2]);
	SendCmnd6(CMND[1]);
}//End of Command_05()

void Command_06(void)
{
	//((06)Single Command x N
	unsigned short N;
	unsigned short i;

	N = CMND[2];
	N = N << 8;
	N += CMND[1];

	for(i=0; i<N; i++)
	{
		SendCmnd6(CMND[0]);
//		mDelayPGC();
	}
}//End of Command_06()

void Command_07(void)
{
	//(07)Receive Data x N
	unsigned short N;
	unsigned short i;
	unsigned short Data;
	
	CDCTxService();
	
	N = CMND[2];
	N = N << 8;
	N += CMND[1];

	for(i=0; i<N; )
	{
		if(mUSBUSARTIsTxTrfReady())
		{
			Data = RcvData1(CMND[0]);
//			mDelayTdly1();
			USB_In_Buffer[0] = Data;
			Data = Data >> 8;
			USB_In_Buffer[1] = Data;

//			USB_In_Buffer[1] = i;
			putUSBUSART(USB_In_Buffer,2);
			SendCmnd6(0b00000110);	// Increment adress
//			mDelayTdly1();
			i++;
		}
		CDCTxService();
	}//end for
}//End of Command_07()

void Command_08(void)
{
	//(07)Receive Data x N
	unsigned char Cmnd1;
	unsigned char Cmnd2;
	unsigned char Cmnd3;
	unsigned char T;
	unsigned char LatchMask;
	unsigned short Data;
	unsigned char sum = 0;
	unsigned char i, iLat;
	unsigned char DataL;

	Cmnd1 = USB_Out_Buffer[0];
	Cmnd2 = USB_Out_Buffer[1];
	Cmnd3 = USB_Out_Buffer[2];
	T = USB_Out_Buffer[3];
	LatchMask = USB_Out_Buffer[4];

	for(i=5, iLat=0; i<13; )
	{
		//Data = USB_Out_Buffer[i++];
		//Data += (unsigned short)USB_Out_Buffer[i++] << 8;
		DataL = USB_Out_Buffer[i++];
		Data = (unsigned short)USB_Out_Buffer[i++];
		Data = Data << 8;
		Data += DataL;
		SendCmnd6(Cmnd1);
		SendData14(Data);
		if(iLat == LatchMask)
		{
			SendCmnd6(Cmnd2);
			mDelay100usx(T);
			if(Cmnd3) SendCmnd6(Cmnd3);
			iLat = 0;
		}
		else
		{
			iLat++;
		}
		SendCmnd6(0b00000110);	//Increment adress command
	}

	for(i=0; i<13; i++)
	{
		sum += USB_Out_Buffer[i];
	}

	USB_In_Buffer[0] = sum;
	putUSBUSART(USB_In_Buffer,1);
}

// Command for pic18

void SendCmnd4(unsigned char Cmnd, unsigned char ID)
{
	// Send Command(4bit) to the device
	unsigned char i;
	mPGC_L();
	mPGD_L();
	for(i=0; i<4; i++)
	{
		mPGC_H();
		if(Cmnd & 0x01)
		{
			mPGD_H();
		}
		else
		{
			mPGD_L();
		}
		mDelayPGC18();
		if(ID<2 || i<3)
			mPGC_L();
		mDelayPGC18();
		Cmnd = Cmnd >> 1;
	}//end for
	mPGD_L();
	mDelayTdly18();
}// End SendCmnd6()

void SendData16(unsigned short Data)
{
	// Send Data(16bit) to the device
	unsigned char i;
	mPGC_L();
	mPGD_L();
	for(i=0; i<16; i++)
	{
		mPGC_H();
		if(Data & 0x0001)
		{
			mPGD_H();
		}
		else
		{
			mPGD_L();
		}
		mDelayPGC18();
		mPGC_L();
		mDelayPGC18();
		Data = Data >> 1;
	}//end for
	mPGD_L();
	mDelayTdly18();
}//End SendData8()

unsigned char RcvData18(unsigned char Cmnd)
{
	// Recieve Data(14bit) from the device after sending a read command(6bit)
	unsigned char i;
	unsigned char RcvData = 0;
	SendCmnd4(Cmnd, 0);
	mPGC_L();
	mPGD_L();
//	mDelayTdly1();

	for(i=0; i<8; i++)
	{
		mPGC_H();
		mDelayPGC();
		mPGC_L();
		mDelayPGC();
	}//End for
	mTRIS_PGD = 1;	// Change Port_PGD to Input
	mDelayTdly18();
	for(i=0; i<8; i++)
	{
		mPGC_H();
		mDelayPGC();
		RcvData = RcvData >> 1;
		if(mPort_PGD)
		{
			RcvData += 0x80;
		}
		mPGC_L();
		mDelayPGC();
	}//End for

	mTRIS_PGD = 0;	// Change Port_PGD to Output
	mDelayTdly18();
	return RcvData;	
}


void Command_09(void)
{
	// [PIC18} Send20
	unsigned char Cmnd;
	unsigned char DataL;
	unsigned short Data;
	unsigned char ID;
	unsigned char T;

	Cmnd = USB_Out_Buffer[0];
	DataL = USB_Out_Buffer[1];
	Data = USB_Out_Buffer[2];
	Data = Data << 8;
	Data += DataL;
	ID = USB_Out_Buffer[3];
	T = USB_Out_Buffer[4];

	SendCmnd4(Cmnd, ID);
	
	if(ID>0)
	{
		mDelay100usx(T);
		if(ID==2)
		{
			mPGC_L();		
		}
		mDelayP10();
	}
	
	SendData16(Data);

	EchoSum(5);

} // End Command_09()

void Command_0B(void)
{
	//(0B)Address Set24
	unsigned short Data;

	// MOVELW AddrU
	Data = mMOVLW;
	Data += (unsigned char)USB_Out_Buffer[2];;
	SendCmnd4(0, 0);
	SendData16(Data);

	// MOVEWF TBLPTRU
	Data = mMOVWF_TBLPTRU;
	SendCmnd4(0, 0);
	SendData16(Data);

	// MOVELW AddrH
	Data = mMOVLW;
	Data += (unsigned char)USB_Out_Buffer[1];
	SendCmnd4(0, 0);
	SendData16(Data);

	// MOVEWF TBLPTRH
	Data = mMOVWF_TBLPTRH;
	SendCmnd4(0, 0);
	SendData16(Data);

	// MOVELW AddrL
	Data = mMOVLW;
	Data += (unsigned char)USB_Out_Buffer[0];
	SendCmnd4(0, 0);
	SendData16(Data);

	// MOVEWF TBLPTRL
	Data = mMOVWF_TBLPTRL;
	SendCmnd4(0, 0);
	SendData16(Data);

	EchoSum(3);
}//End of Command_0B()

void Command_0C(void)
{
	//(0C)Table Read N-Byte
	unsigned short N;
	unsigned short i;
	unsigned char Data;
	
	CDCTxService();
	
	N = USB_Out_Buffer[1];
	N = N << 8;
	N += USB_Out_Buffer[0];

	for(i=0; i<N; )
	{
		if(mUSBUSARTIsTxTrfReady())
		{
			//Table Read, post-increment 1001
			Data = RcvData18(0b00001001);
			USB_In_Buffer[0] = Data;
			putUSBUSART(USB_In_Buffer,1);
			i++;
		}
		CDCTxService();
	}//end for
}//End of Command_0C()

void Command_0D(void)
{
	//(0D)Write Data 8Byte
	unsigned char T;
	unsigned char dataL;
	unsigned short data;
	unsigned char i;

	T = USB_Out_Buffer[0];
	for(i=1; i<9; )
	{
		if(T>0 && i>=7)
		{
			SendCmnd4(mTWrite_StrtPrg, 0);
		}
		else
		{
			SendCmnd4(mTWrite_PstInc2, 0);
		}
		dataL = USB_Out_Buffer[i++];
		data = USB_Out_Buffer[i++];
		data = data << 8;
		data += dataL;
		SendData16(data);
	}
	if(T>0)
	{
		SendCmnd4(0, 2);
		mDelay100usx(T);
		mPGC_L();
		mDelayP10();		
		SendData16(0);
	}
	EchoSum(9);
}//End Command_0D(void)

void Command_0E(void)
{
	//(0E)EEPROM Address Set
	unsigned short Data;

	SendCmnd4(0, 0);
	SendData16(mBCF_EECON1_EEPGD);

	SendCmnd4(0, 0);
	SendData16(mBCF_EECON1_CFGS);

	Data = mMOVLW;
	Data += (unsigned char)USB_Out_Buffer[0];;
	SendCmnd4(0, 0);
	SendData16(Data);

	SendCmnd4(0, 0);
	SendData16(mMOVWF_EEADR);

	Data = mMOVLW;
	Data += (unsigned char)USB_Out_Buffer[1];;
	SendCmnd4(0, 0);
	SendData16(Data);

	SendCmnd4(0, 0);
	SendData16(mMOVWF_EEADRH);

	EchoSum(2);
}	//End Command_0E(void)

void Command_0F(unsigned short N)
{
	//(0F)EEPROM Read N-Byte
	//unsigned short N;
	unsigned short i;
	unsigned char Data;
	
	CDCTxService();
	
	//N = Cmnd;
	//if(N==0)
	//	N = 256;

	i=0;
	//for(i=0; i<N; )
	do{
		if(mUSBUSARTIsTxTrfReady())
		{
			SendCmnd4(0, 0);
			SendData16(mBSF_EECON1_RD);

			SendCmnd4(0, 0);
			SendData16(mMOVF_EEDATA_W);

			SendCmnd4(0, 0);
			SendData16(mMOVWF_TABLAT);

			SendCmnd4(0, 0);
			SendData16(0x0000);

			Data = RcvData18(0x02);
			USB_In_Buffer[0] = Data;

			SendCmnd4(0, 0);
			SendData16(mINCF_EEADR);

			putUSBUSART(USB_In_Buffer,1);
			i++;
		}
		CDCTxService();
	}while(i!=N);
	//end for
}



void Command_10(unsigned char Cmnd)
{
	//(10)EEPROM Write 1Byte
	unsigned short Data;
	unsigned char dEECON1;
	unsigned char Itr = 0;

	Data = mMOVLW;
	Data += Cmnd;

	SendCmnd4(0, 0);
	SendData16(Data);

	SendCmnd4(0, 0);
	SendData16(mMOVWF_EEDATA);

	SendCmnd4(0, 0);
	SendData16(mBSF_EECON1_WREN);

	SendCmnd4(0, 0);
	SendData16(mMOVLW_0x55);

	SendCmnd4(0, 0);
	SendData16(mMOVWF_EECON2);

	SendCmnd4(0, 0);
	SendData16(mMOVLW_0xAA);

	SendCmnd4(0, 0);
	SendData16(mMOVWF_EECON2);

	SendCmnd4(0, 0);
	SendData16(mBSF_EECON1_WR);

	SendCmnd4(0, 0);
	SendData16(0x0000);

	SendCmnd4(0, 0);
	mDelay10u();	// 10 usec
//	mDelay100usx(50);	// 5 msec
	SendData16(0x0000);

//	mDelay100usx(50);	// 5 msec

	do{
		Delay10TCYx(1);	//1 usec

		SendCmnd4(0, 0);
		SendData16(mMOVF_EECON1_W);

		SendCmnd4(0, 0);
		SendData16(mMOVWF_TABLAT);

		SendCmnd4(0, 0);
		SendData16(0x0000);

		dEECON1 = RcvData18(0x02);
		Itr++;
	}while((dEECON1 & 0b00000010) && (Itr < mMaxItr)); 

	mDelayP10();

	SendCmnd4(0, 0);
	SendData16(mBCF_EECON1_WREN);

	SendCmnd4(0, 0);
	SendData16(mINCF_EEADR);

	USB_In_Buffer[0] = Cmnd;
	putUSBUSART(USB_In_Buffer,1);

}

void EchoSum(unsigned char N)
{
	unsigned char sum = 0;
	unsigned char i;

	for(i=0; i<N; i++)
	{
		sum += (unsigned char)USB_Out_Buffer[i];
	}

	USB_In_Buffer[0] = sum;	// check sum
	putUSBUSART(USB_In_Buffer,1);
}
